/*
 * Copyright (c) 2008 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup net
 * @{
 */

/** @file
 */

#define NET_NETIF_COUNT		7
#define NET_NETWORKING_COUNT	4
#define NET_LL_COUNT		5
#define NET_ETHERNET_COUNT	0
#define NET_IL_COUNT		2
#define NET_IP_COUNT		4
#define NET_ARP_COUNT		0
#define NET_RARP_COUNT		0
#define NET_ICMP_COUNT		0
#define NET_UDP_COUNT		0
#define NET_TCP_COUNT		0
#define NET_SOCKET_COUNT	0

#define NET_FIRST		2000

#define NET_NETIF_FIRST		NET_FIRST
#define NET_NETIF_LAST		( NET_NETIF_FIRST + NET_NETIF_COUNT )

#define NET_NETWORKING_FIRST	( NET_NETIF_LAST + 0 )
#define NET_NETWORKING_LAST	( NET_NETWORKING_FIRST + NET_NETWORKING_COUNT )

#define NET_LL_FIRST		( NET_NETWORKING_LAST + 0 )
#define NET_LL_LAST		( NET_LL_FIRST + NET_LL_COUNT )
#define NET_ETHERNET_FIRST	( NET_LL_LAST + 0 )
#define NET_ETHERNET_LAST	( NET_ETHERNET_FIRST + NET_ETHERNET_COUNT )

#define NET_IL_FIRST		( NET_ETHERNET_LAST + 0 )
#define NET_IL_LAST		( NET_IL_FIRST + NET_IL_COUNT )
#define NET_IP_FIRST		( NET_IL_LAST + 0 )
#define NET_IP_LAST		( NET_IP_FIRST + NET_IP_COUNT )

#define NET_ARP_FIRST		( NET_IP_LAST + 0 )
#define NET_ARP_LAST		( NET_ARP_FIRST + NET_ARP_COUNT )
#define NET_RARP_FIRST		( NET_ARP_LAST + 0 )
#define NET_RARP_LAST		( NET_RARP_FIRST + NET_RARP_COUNT )
#define NET_ICMP_FIRST		( NET_RARP_LAST + 0 )
#define NET_ICMP_LAST		( NET_ICMP_FIRST + NET_ICMP_COUNT )
#define NET_UDP_FIRST		( NET_ICMP_LAST + 0 )
#define NET_UDP_LAST		( NET_UDP_FIRST + NET_UDP_COUNT )
#define NET_TCP_FIRST		( NET_UDP_LAST + 0 )
#define NET_TCP_LAST		( NET_TCP_FIRST + NET_TCP_COUNT )

#define NET_SOCKET_FIRST	( NET_TCP_LAST + 0 )
#define NET_SOCKET_LAST		( NET_SOCKET_FIRST + NET_SOCKET_COUNT )

#define NET_LAST		NET_SOCKET_LAST

#define NET_COUNT		( NET_LAST - NET_FIRST )

#define IS_IN_INTERVAL( item, first_inclusive, last_exclusive )	((( item ) >= ( first_inclusive )) && (( item ) < ( last_exclusive )))

#define IS_NET_MESSAGE( call )			IS_IN_INTERVAL( IPC_GET_METHOD( call ), NET_FIRST, NET_LAST )
#define IS_NET_NETWORKING_MESSAGE( call )	IS_IN_INTERVAL( IPC_GET_METHOD( call ), NET_NETWORKING_FIRST, NET_NETWORKING_LAST )
#define IS_NET_LL_MESSAGE( call )		IS_IN_INTERVAL( IPC_GET_METHOD( call ), NET_LL_FIRST, NET_LL_LAST )
#define IS_NET_ETHERNET_MESSAGE( call )		IS_IN_INTERVAL( IPC_GET_METHOD( call ), NET_ETHERNET_FIRST, NET_SOCKET_LAST )
#define IS_NET_IL_MESSAGE( call )		IS_IN_INTERVAL( IPC_GET_METHOD( call ), NET_IL_FIRST, NET_IL_LAST )
#define IS_NET_IP_MESSAGE( call )		IS_IN_INTERVAL( IPC_GET_METHOD( call ), NET_IP_FIRST, NET_IP_LAST )
#define IS_NET_ARP_MESSAGE( call )		IS_IN_INTERVAL( IPC_GET_METHOD( call ), NET_ARP_FIRST, NET_ARP_LAST )
#define IS_NET_RARP_MESSAGE( call )		IS_IN_INTERVAL( IPC_GET_METHOD( call ), NET_RARP_FIRST, NET_RARP_LAST )
#define IS_NET_UDP_MESSAGE( call )		IS_IN_INTERVAL( IPC_GET_METHOD( call ), NET_UDP_FIRST, NET_UDP_LAST )
#define IS_NET_TCP_MESSAGE( call )		IS_IN_INTERVAL( IPC_GET_METHOD( call ), NET_TCP_FIRST, NET_TCP_LAST )
#define IS_NET_SOCKET_MESSAGE( call )		IS_IN_INTERVAL( IPC_GET_METHOD( call ), NET_SOCKET_FIRST, NET_SOCKET_LAST )

typedef enum {
	NET_NETIF_PROBE = NET_NETIF_FIRST,
	NET_NETIF_PROBE_AUTO,
	NET_NETIF_REGISTER,
	NET_NETIF_SEND,
	NET_NETIF_START,
	NET_NETIF_STATS,
	NET_NETIF_STOP,
	NET_NETWORKING_DEVICE = NET_NETWORKING_FIRST,
	NET_NETWORKING_GET_CONFIGURATION,
	NET_NETWORKING_GET_DEVICE_CONFIGURATION,
	NET_NETWORKING_STARTUP,
	NET_LL_DEVICE = NET_LL_FIRST,
	NET_LL_DEVICE_STATE_CHANGED,
	NET_LL_MTU_CHANGED,
	NET_LL_RECEIVED,
	NET_LL_REGISTER,
	NET_IL_DEVICE = NET_IL_FIRST,
	NET_IL_DEVICE_STATE_CHANGED,
	NET_IP_CONFIGURATION_DHCP = NET_IP_FIRST,
	NET_IP_CONFIGURATION_STATIC,
	NET_IP_ECHO,
	NET_IP_TCP_REGISTER
} net_message;

/** @}
 */
